package de.kif.backend.route

import de.kif.backend.Configuration
import de.kif.backend.repository.PostRepository
import de.kif.backend.repository.ScheduleRepository
import de.kif.backend.view.respondMain
import de.kif.common.model.Schedule
import io.ktor.routing.Route
import io.ktor.routing.get
import kotlinx.css.CSSBuilder
import kotlinx.css.Color
import kotlinx.html.*
import java.util.*

fun Route.board() {
    get("/brett") {
        val postList = PostRepository.all().asReversed()
        val scheduleList = ScheduleRepository.all().map {
            it to it.getAbsoluteStartTime() * 60
        }.sortedBy { it.second }

        val referenceTime = Configuration.Schedule.referenceDate.time / 1000
        val now = referenceTime - (Date().time / 1000)

        respondMain(true, true) { theme ->
            content {
                div("board") {
                    div("board-schedules") {
                        attributes["data-reference"] = referenceTime.toString()

                        div("board-header") {
                            +"AKs"
                        }

                        div("board-card board-schedule-box") {
                            div("board-card-header") {
                                +"Running"
                            }
                            table {
                            for ((schedule, time) in scheduleList) {
                                createBoardSchedule(schedule, time)
                            }
                            }
                        }

                        div("board-card board-schedule-box") {
                            div("board-card-header") {
                                +"Upcoming"
                            }
                            table {
                                for ((schedule, time) in scheduleList) {
                                    createBoardSchedule(schedule, time)
                                }
                            }
                        }
                    }

                    div("board-posts") {
                        div("board-header") {
                            +"News"
                        }
                        for (post in postList) {
                            createPost(post, false, "board-card board-post")
                        }
                    }

                    div("board-twitter") {
                        div("board-header") {
                            +"Tweets"
                        }
                        div("board-card") {
                            unsafe {
                                raw("""
                                <a
                                    class="twitter-timeline"
                                    href="${Configuration.Twitter.timeline}"
                                    data-chrome="transparent noheader nofooter"
                                    data-theme="${if (theme.dark) "dark" else "light"}"
                                    data-link-color="${theme.primaryColor}"
                                    data-cards="hidden"
                                    data-lang="de"
                                    data-dnt="true"
                                >Twitter wall</a>
                                <script async src="https://platform.twitter.com/widgets.js" charset="utf-8"></script>
                            """.trimIndent())
                            }
                        }
                    }
                }
            }
        }
    }
}

private fun TABLE.createBoardSchedule(schedule: Schedule, time: Int) {
    tr("board-schedule") {
        attributes["data-id"] = schedule.id.toString()

        td("board-schedule-color") {
            span {
                attributes["style"] = CSSBuilder().apply {
                    val c = schedule.workGroup.track?.color
                    if (c != null) {
                        backgroundColor = Color(c.toString())
                    }
                }.toString()
            }
        }

        td("board-schedule-time") {
            attributes["data-time"] = time.toString()
            attributes["data-duration"] = schedule.workGroup.length.toString()


            val startTime = (time % MINUTES_OF_DAY).let {
                if (it < 0) it + MINUTES_OF_DAY else it
            }
            val sm = (startTime % 60).toString().padStart(2, '0')
            val sh = (startTime / 60).toString().padStart(2, '0')
            val startTimeString = "$sh:$sm"

            val endTime = ((time + schedule.workGroup.length) % MINUTES_OF_DAY).let {
                if (it < 0) it + MINUTES_OF_DAY else it
            }
            val em = (endTime % 60).toString().padStart(2, '0')
            val eh = (endTime / 60).toString().padStart(2, '0')
            val endTimeString = "$eh:$em"

            +"$startTimeString - $endTimeString"
        }

        td("board-schedule-name") {
            +schedule.workGroup.name
        }

        td("board-schedule-room") {
            +schedule.room.name
        }
    }
}
